"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var couchbase_plugin_common_1 = require("./couchbase-plugin.common");
var utils = require("tns-core-modules/utils/utils");
var Couchbase = (function (_super) {
    __extends(Couchbase, _super);
    function Couchbase(name) {
        var _this = _super.call(this, name) || this;
        _this.config = new com.couchbase.lite.DatabaseConfiguration(utils.ad.getApplicationContext());
        _this.android = new com.couchbase.lite.Database(name, _this.config);
        return _this;
    }
    Couchbase.prototype.createDocument = function (data, documentId) {
        try {
            var doc = void 0;
            if (documentId) {
                doc = new com.couchbase.lite.MutableDocument(documentId);
            }
            else {
                doc = new com.couchbase.lite.MutableDocument();
            }
            var keys = Object.keys(data);
            for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
                var key = keys_1[_i];
                var item = data[key];
                this.serialize(item, doc, key);
            }
            this.android.save(doc);
            return doc.getId();
        }
        catch (e) {
            return null;
        }
    };
    Couchbase.prototype.getDocument = function (documentId) {
        try {
        }
        catch (e) {
        }
        var doc = this.android.getDocument(documentId);
        var keys = doc.getKeys();
        var size = keys.size();
        var object = {};
        for (var i = 0; i < size; i++) {
            var key = keys.get(i);
            var item = doc.getValue(key);
            var newItem = {};
            newItem[key] = item;
            object = Object.assign(object, newItem);
        }
        return object;
    };
    Couchbase.prototype.fromISO8601UTC = function (date) {
        var dateFormat = new java.text.SimpleDateFormat('yyyy-MM-dd\'T\'HH:mm:ss.SSSXXX');
        var tz = java.util.TimeZone.getTimeZone('UTC');
        dateFormat.setTimeZone(tz);
        return dateFormat.parse(date);
    };
    Couchbase.prototype.toISO8601UTC = function (date) {
        var dateFormat = new java.text.SimpleDateFormat('yyyy-MM-dd\'T\'HH:mm:ss.SSSXXX');
        var tz = java.util.TimeZone.getTimeZone('UTC');
        dateFormat.setTimeZone(tz);
        return dateFormat.format(date);
    };
    Couchbase.prototype.updateDocument = function (documentId, data) {
        try {
            var origin = this.android.getDocument(documentId);
            if (origin) {
                var doc = origin.toMutable();
                var keys = Object.keys(data);
                for (var _i = 0, keys_2 = keys; _i < keys_2.length; _i++) {
                    var key = keys_2[_i];
                    var item = data[key];
                    this.serialize(item, doc, key);
                }
                this.android.save(doc);
            }
        }
        catch (e) {
            console.error(e.message);
        }
    };
    Couchbase.prototype.serializeObject = function (item, object, key) {
        var _this = this;
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    object.setDate(key, this.fromISO8601UTC(item.toISOString()));
                    return;
                }
                if (Array.isArray(item)) {
                    var array_1 = new com.couchbase.lite.MutableArray();
                    item.forEach(function (data) {
                        _this.serializeArray(data, array_1);
                    });
                    object.setArray(key, array_1);
                    return;
                }
                var nativeObject_1 = new com.couchbase.lite.MutableDictionary();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, nativeObject_1, itemKey);
                });
                object.setDictionary(key, object);
                break;
            case 'number':
                object.setInt(key, item);
                break;
            case 'boolean':
                object.setBoolean(key, item);
                break;
            default:
                object.setValue(key, item);
        }
    };
    Couchbase.prototype.serializeArray = function (item, array) {
        var _this = this;
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    array.addDate(this.fromISO8601UTC(item.toISOString()));
                    return;
                }
                if (Array.isArray(item)) {
                    var nativeArray_1 = new com.couchbase.lite.MutableArray();
                    item.forEach(function (data) {
                        _this.serializeArray(data, nativeArray_1);
                    });
                    array.addArray(nativeArray_1);
                    return;
                }
                var object_1 = new com.couchbase.lite.MutableDictionary();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, object_1, itemKey);
                });
                array.addDictionary(object_1);
                break;
            case 'number':
                array.addInt(item);
                break;
            case 'boolean':
                array.addBoolean(item);
                break;
            default:
                array.addValue(item);
        }
    };
    Couchbase.prototype.serialize = function (item, doc, key) {
        var _this = this;
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    doc.setDate(key, this.fromISO8601UTC(item.toISOString()));
                    return;
                }
                if (Array.isArray(item)) {
                    var array_2 = new com.couchbase.lite.MutableArray();
                    item.forEach(function (data) {
                        _this.serializeArray(data, array_2);
                    });
                    doc.setArray(key, array_2);
                    return;
                }
                var object_2 = new com.couchbase.lite.MutableDictionary();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, object_2, itemKey);
                });
                doc.setDictionary(key, object_2);
                break;
            case 'number':
                doc.setInt(key, item);
                break;
            case 'boolean':
                doc.setBoolean(key, item);
                break;
            default:
                doc.setValue(key, item);
        }
    };
    Couchbase.prototype.deleteDocument = function (documentId) {
        try {
            var doc = this.android.getDocument(documentId);
            return this.android.delete(doc);
        }
        catch (e) {
            console.error(e.message);
            return false;
        }
    };
    Couchbase.prototype.destroyDatabase = function () {
        try {
            this.android.delete();
        }
        catch (e) {
            console.error(e.message);
        }
    };
    Couchbase.prototype.query = function (query) {
        if (query === void 0) { query = { select: [] }; }
        var items = [];
        var select = [];
        if (!query.select || query.select.length === 0) {
            select.push(com.couchbase.lite.SelectResult.all());
        }
        else {
            query.select.forEach(function (item) {
                if (item === couchbase_plugin_common_1.QueryMeta.ID) {
                    select.push(com.couchbase.lite.Meta.id);
                }
                else {
                    select.push(com.couchbase.lite.Expression.property(item));
                }
            });
        }
        var queryBuilder = com.couchbase.lite.QueryBuilder.select(select);
        if (query.from) {
            var db = new Couchbase(query.from);
            queryBuilder = queryBuilder.from(com.couchbase.lite.DataSource.database(db.android));
        }
        else {
            queryBuilder = queryBuilder.from(com.couchbase.lite.DataSource.database(this.android));
        }
        var nativeQuery;
        if (query.where) {
            for (var _i = 0, _a = query.where; _i < _a.length; _i++) {
                var item = _a[_i];
                if (item === couchbase_plugin_common_1.QueryLogicalOperator.AND) {
                    if (!nativeQuery)
                        break;
                }
                else if (item === couchbase_plugin_common_1.QueryLogicalOperator.OR) {
                    if (!nativeQuery)
                        break;
                }
                else {
                    if (item) {
                        switch (item.comparison) {
                            case 'equalTo':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).equalTo(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'add':
                                break;
                            case 'between':
                                break;
                            case 'collate':
                                break;
                            case 'divide':
                                break;
                            case 'greaterThan':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).greaterThan(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'greaterThanOrEqualTo':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).greaterThanOrEqualTo(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'in':
                                var inArray = [];
                                if (Array.isArray(item.value)) {
                                    for (var _b = 0, _c = item.value; _b < _c.length; _b++) {
                                        var exp_1 = _c[_b];
                                        inArray.push(com.couchbase.lite.Expression.value(exp_1));
                                    }
                                }
                                else {
                                    inArray.push(com.couchbase.lite.Expression.value(item.value));
                                }
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).in(inArray);
                                break;
                            case 'is':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).is(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'isNot':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).isNot(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'isNullOrMissing':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).isNullOrMissing();
                                break;
                            case 'lessThan':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).lessThan(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'lessThanOrEqualTo':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).lessThanOrEqualTo(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'like':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).like(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'modulo':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).modulo(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'multiply':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).multiply(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'notEqualTo':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).notEqualTo(com.couchbase.lite.Expression.value(item.value));
                                break;
                            case 'notNullOrMissing':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).notNullOrMissing();
                                break;
                            case 'regex':
                                nativeQuery = com.couchbase.lite.Expression.property(item.property).regex(com.couchbase.lite.Expression.value(item.value));
                                break;
                        }
                    }
                }
            }
            if (nativeQuery) {
                queryBuilder = queryBuilder.where(nativeQuery);
            }
        }
        if (query.groupBy) {
            var groupBy = [];
            for (var _d = 0, _e = query.groupBy; _d < _e.length; _d++) {
                var prop = _e[_d];
                groupBy.push(com.couchbase.lite.Expression.property(prop));
            }
            if (groupBy.length > 0) {
                queryBuilder = queryBuilder.groupBy(groupBy);
            }
        }
        if (query.order) {
            var orderBy = [];
            for (var _f = 0, _g = query.order; _f < _g.length; _f++) {
                var item = _g[_f];
                switch (item.direction) {
                    case 'desc':
                        orderBy.push(com.couchbase.lite.Ordering.property(item.property).descending());
                        break;
                    default:
                        orderBy.push(com.couchbase.lite.Ordering.property(item.property).ascending());
                        break;
                }
            }
            if (orderBy.length > 0) {
                queryBuilder = queryBuilder.orderBy(orderBy);
            }
        }
        if (query.limit && typeof query.limit === 'number') {
            queryBuilder = queryBuilder.limit(com.couchbase.lite.Expression.value(query.limit));
        }
        var result = queryBuilder.execute().allResults();
        var size = result.size();
        for (var i = 0; i < size; i++) {
            var item = result.get(i);
            var keys = item.getKeys();
            var keysSize = keys.size();
            var obj = {};
            for (var keyId = 0; keyId < keysSize; keyId++) {
                var key = keys.get(keyId);
                var nativeItem = item.getValue(key);
                if (nativeItem.getClass().getName() === 'com.couchbase.lite.Dictionary') {
                    var cblKeys = nativeItem.getKeys();
                    var cblKeysSize = cblKeys.size();
                    for (var cblKeysId = 0; cblKeysId < cblKeysSize; cblKeysId++) {
                        var cblKey = cblKeys.get(cblKeysId);
                        obj[cblKey] = nativeItem.getValue(cblKey);
                    }
                }
            }
            items.push(obj);
        }
        return items;
    };
    Couchbase.prototype.createPullReplication = function (remoteUrl, username, password) {
        var uri = new com.couchbase.lite.URLEndpoint(new java.net.URI(remoteUrl));
        var repConfig = new com.couchbase.lite.ReplicatorConfiguration(this.android, uri);
        repConfig.setReplicatorType(com.couchbase.lite.ReplicatorConfiguration.ReplicatorType.PULL);
        if (username && password) {
            repConfig.setAuthenticator(new com.couchbase.lite.BasicAuthenticator(username, password));
        }
        var replicator = new com.couchbase.lite.Replicator(repConfig);
        return new Replicator(replicator);
    };
    Couchbase.prototype.createPushReplication = function (remoteUrl, username, password) {
        var uri = new com.couchbase.lite.URLEndpoint(new java.net.URI(remoteUrl));
        var repConfig = new com.couchbase.lite.ReplicatorConfiguration(this.android, uri);
        repConfig.setReplicatorType(com.couchbase.lite.ReplicatorConfiguration.ReplicatorType.PUSH);
        if (username && password) {
            repConfig.setAuthenticator(new com.couchbase.lite.BasicAuthenticator(username, password));
        }
        var replicator = new com.couchbase.lite.Replicator(repConfig);
        return new Replicator(replicator);
    };
    Couchbase.prototype.addDatabaseChangeListener = function (callback) {
        var listener = co.fitcom.fancycouchbase.TNSDatabaseChangeListener.extend({
            onChange: function (changes) {
                if (callback && typeof callback === 'function') {
                    var ids = [];
                    var documentIds = changes.getDocumentIDs();
                    var size = documentIds.size();
                    for (var i = 0; i < size; i++) {
                        var item = documentIds.get(i);
                        ids.push(item);
                    }
                    callback(ids);
                }
            }
        });
        this.android.addChangeListener(new listener());
    };
    return Couchbase;
}(couchbase_plugin_common_1.Common));
exports.Couchbase = Couchbase;
var Replicator = (function (_super) {
    __extends(Replicator, _super);
    function Replicator(replicator) {
        return _super.call(this, replicator) || this;
    }
    Replicator.prototype.start = function () {
        this.replicator.start();
    };
    Replicator.prototype.stop = function () {
        this.replicator.stop();
    };
    Replicator.prototype.isRunning = function () {
        return (this.replicator.getStatus().getActivityLevel() ===
            com.couchbase.lite.AbstractReplicator.ActivityLevel.BUSY);
    };
    Replicator.prototype.setContinuous = function (isContinuous) {
        this.replicator.getConfig().setContinuous(isContinuous);
    };
    return Replicator;
}(couchbase_plugin_common_1.ReplicatorBase));
exports.Replicator = Replicator;
//# sourceMappingURL=couchbase-plugin.android.js.map